@@ -19,13 +19,16 @@
 
 
 #include "plaintextextractor.h"
+
 #include <QFile>
+#include <QTextCodec>
+#include <QDebug>
+
 #include <fstream>
-#include <KService>
 
 using namespace KFileMetaData;
 
-PlainTextExtractor::PlainTextExtractor(QObject* parent, const QVariantList&)
+PlainTextExtractor::PlainTextExtractor(QObject* parent)
     : ExtractorPlugin(parent)
 {
 
@@ -33,7 +36,7 @@ PlainTextExtractor::PlainTextExtractor(QObject* parent, const QVariantList&)
 
 QStringList PlainTextExtractor::mimetypes() const
 {
-    return QStringList() << QLatin1String("text/");
+    return QStringList() << QStringLiteral("text/");
 }
 
 void PlainTextExtractor::extract(ExtractionResult* result)
@@ -46,17 +49,26 @@ void PlainTextExtractor::extract(ExtractionResult* result)
         return;
     }
 
+    result->addType(Type::Text);
+    if (!(result->inputFlags() & ExtractionResult::ExtractPlainText)) {
+        return;
+    }
+
+    QTextCodec* codec = QTextCodec::codecForLocale();
     while (std::getline(fstream, line)) {
         QByteArray arr = QByteArray::fromRawData(line.c_str(), line.size());
-        result->append(QString::fromUtf8(arr));
+
+        QTextCodec::ConverterState state;
+        QString text = codec->toUnicode(arr.constData(), arr.size(), &state);
+
+        if (state.invalidChars > 0) {
+            qDebug() << "Invalid encoding. Ignoring" << result->inputUrl();
+            return;
+        }
+        result->append(text);
 
         lines += 1;
     }
 
     result->add(Property::LineCount, lines);
-    result->addType(Type::Text);
 }
-
-K_PLUGIN_FACTORY(factory, registerPlugin<PlainTextExtractor>();)
-
-#include "plaintextextractor.moc"
